@@ -120,6 +120,9 @@ ghex_window_close(GHexWindow *win)
 	if (converter_get)
 		gtk_widget_set_sensitive(converter_get, FALSE);
 
+    if (win->advanced_find_dialog)
+        delete_advanced_find_dialog(win->advanced_find_dialog);
+
     gtk_widget_destroy(GTK_WIDGET(win));
 
     if (doc->views == NULL) /* If we have destroyed the last view */
@@ -238,6 +241,11 @@ ghex_window_destroy(GtkObject *object)
                                                  win);
             win->gh = NULL;
         }
+        if (win->dialog)
+        {
+            g_object_unref (G_OBJECT(win->dialog));
+            win->dialog = NULL;
+        }
 
         window_list = g_list_remove(window_list, win);
 
@@ -405,6 +413,22 @@ ghex_window_listener (BonoboUIComponent           *uic,
         }
         ghex_window_sync_char_table_item(win, atoi(state));
         return;
+    }
+    else if (strcmp(path, "TypeDialog") == 0)
+    {
+        if (!win->dialog)
+            return;
+        if (state && atoi(state)) {
+            if (!GTK_WIDGET_VISIBLE(win->dialog_widget))
+            {
+                gtk_widget_show(win->dialog_widget);
+            }
+        }
+        else if (GTK_WIDGET_VISIBLE(win->dialog_widget))
+        {
+            gtk_widget_hide(GTK_WIDGET(win->dialog_widget));
+        }
+        return;
     }
 	if (!state || !atoi (state) || (win->gh == NULL))
 		return;
@@ -465,12 +489,16 @@ ghex_window_new(void)
                                       ghex_window_listener, win);
 	bonobo_ui_component_add_listener (uic, "CharacterTable",
                                       ghex_window_listener, win);
+	bonobo_ui_component_add_listener (uic, "TypeDialog",
+                                      ghex_window_listener, win);
     bonobo_ui_component_set_prop (uic, "/commands/Converter", "state",
                                   (converter && GTK_WIDGET_VISIBLE(converter->window))?"1":"0",
                                   NULL);
     bonobo_ui_component_set_prop (uic, "/commands/CharacterTable", "state",
                                   (char_table && GTK_WIDGET_VISIBLE(char_table))?"1":"0",
                                   NULL);
+    bonobo_ui_component_set_prop (uic, "/commands/TypeDialog", "state",
+                                  "1", NULL);
 	bonobo_ui_component_set_prop (uic, "/status", "hidden", "0", NULL);
 
     ghex_window_set_sensitivity(win);
@@ -566,15 +594,26 @@ static void
 cursor_moved_cb(GtkHex *gtkhex, gpointer user_data)
 {
 	static gchar *cursor_pos, *format;
+    int i;
+    int current_pos;
+    HexDialogVal64 val;
 	GHexWindow *win = GHEX_WINDOW(user_data);
 
+    current_pos = gtk_hex_get_cursor(gtkhex);
+
 	if((format = g_strdup_printf(_("Offset: %s"), offset_fmt)) != NULL) {
-		if((cursor_pos = g_strdup_printf(format, gtk_hex_get_cursor(gtkhex))) != NULL) {
+		if((cursor_pos = g_strdup_printf(format, current_pos)) != NULL) {
 			ghex_window_show_status(win, cursor_pos);
             g_free(cursor_pos);
 		}
 		g_free(format);
 	}
+    for (i = 0; i < 8; i++)
+    {
+        /* returns  0 on buffer overflow, which is what we want */
+        val.v[i] = gtk_hex_get_byte(gtkhex, current_pos+i);
+    }
+    hex_dialog_updateview(win->dialog, &val);
 }
 
 gboolean
@@ -583,7 +622,9 @@ ghex_window_load(GHexWindow *win, const gchar *filename)
     gchar *full_path;
     HexDocument *doc;
     GtkWidget *gh;
+    GtkWidget *vbox;
     const GList *window_list;
+    gchar *state;
 
     g_return_val_if_fail(win != NULL, FALSE);
     g_return_val_if_fail(GHEX_IS_WINDOW(win), FALSE);
@@ -611,7 +652,24 @@ ghex_window_load(GHexWindow *win, const gchar *filename)
     g_signal_connect(G_OBJECT(gh), "cursor_moved",
                      G_CALLBACK(cursor_moved_cb), win);
     gtk_widget_show(gh);
-    
+
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_widget_show(vbox);
+    gtk_box_pack_start(GTK_BOX(vbox), gh, TRUE, TRUE, GNOME_PAD);
+
+    win->dialog = hex_dialog_new();
+    win->dialog_widget = hex_dialog_getview(win->dialog);
+    gtk_box_pack_start(GTK_BOX(vbox), win->dialog_widget, FALSE, FALSE, GNOME_PAD);
+    state = bonobo_ui_component_get_prop (win->uic, "/commands/TypeDialog", "state", NULL);
+    if ((state && atoi(state)) || !state)
+    {
+      gtk_widget_show(win->dialog_widget);
+    }
+    else
+    {
+      gtk_widget_hide(win->dialog_widget);
+    }
+
     if(win->gh) {
         window_list = ghex_window_get_list();
         while(window_list) {
@@ -626,7 +684,7 @@ ghex_window_load(GHexWindow *win, const gchar *filename)
                                              ghex_window_doc_changed,
                                              win);
     }
-    bonobo_window_set_contents(BONOBO_WINDOW(win), gh);
+    bonobo_window_set_contents(BONOBO_WINDOW(win), vbox);
     win->gh = GTK_HEX(gh);
     win->changed = FALSE;
 
@@ -639,6 +697,8 @@ ghex_window_load(GHexWindow *win, const gchar *filename)
     ghex_window_sync_group_type(win);
     ghex_window_set_doc_name(win, win->gh->document->path_end);
     ghex_window_set_sensitivity(win);
+
+    gtk_signal_emit_by_name(GTK_OBJECT(gh), "cursor_moved");
    
     return TRUE;
 }
@@ -930,6 +990,8 @@ ghex_window_save_as(GHexWindow *win)
 	HexDocument *doc;
 	GtkWidget *file_sel;
 	gboolean resp, ret_val = TRUE;
+	const gchar *filename;
+	gboolean dir_flag = FALSE;
 
 	if(win->gh == NULL)
 		return ret_val;
@@ -956,11 +1018,32 @@ ghex_window_save_as(GHexWindow *win)
 	gtk_window_position (GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);
 	gtk_widget_show (file_sel);
 
-	gtk_main();
+	do {
+		gtk_main();
+		filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_sel));
+		if (g_file_test(filename,G_FILE_TEST_IS_DIR)) {
+			gint name_len;
+			gchar *dir_name;
+
+			name_len = strlen (filename);
+			if (name_len < 1 || filename [name_len - 1] != '/')	{
+				dir_name = g_strconcat (filename, "/", NULL);
+			} else  {
+				dir_name = g_strdup (filename);
+			}
+			gtk_file_selection_set_filename (file_sel, dir_name);
+			g_free (dir_name);
+			dir_flag = TRUE;
+		}
+		else  {
+	     dir_flag = FALSE;
+		}
+	} while (resp && dir_flag);	   
+
+
 
 	if(resp) {
 		FILE *file;
-		const gchar *filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_sel));
 		gchar *flash;
 		int i;
         gint save = 0;
@@ -980,6 +1063,9 @@ ghex_window_save_as(GHexWindow *win)
                                                            GNOME_STOCK_BUTTON_YES,
                                                            GNOME_STOCK_BUTTON_NO,
                                                            NULL));
+			 gtk_window_set_type_hint (GTK_WINDOW (mbox), GDK_WINDOW_TYPE_HINT_DIALOG);
+			 gtk_window_set_transient_for (GTK_WINDOW (mbox), GTK_WINDOW (file_sel));
+
             g_free(msg);
             ret_val = (ask_user(mbox) == 0);
         }
@@ -1027,12 +1113,28 @@ ghex_window_save_as(GHexWindow *win)
 	return ret_val;
 }
 
+gboolean
+ghex_window_uri_exists (const gchar* text_uri)
+{
+	GnomeVFSURI *uri;
+	gboolean res;
+
+	g_return_val_if_fail (text_uri != NULL, FALSE);
+	uri = gnome_vfs_uri_new (text_uri);
+	g_return_val_if_fail (uri != NULL, FALSE);
+	res = gnome_vfs_uri_exists (uri);
+
+	gnome_vfs_uri_unref (uri);
+	return res;
+}
+
 gboolean
 ghex_window_ok_to_close(GHexWindow *win)
 {
 	static char msg[MESSAGE_LEN + 1];
 	GtkWidget *mbox;
 	gint reply;
+	gboolean uri_exists;
 	GtkWidget *save_btn;
     HexDocument *doc;
 
@@ -1040,8 +1142,8 @@ ghex_window_ok_to_close(GHexWindow *win)
         return TRUE;
 
     doc = win->gh->document;
-
-    if(!hex_document_has_changed(doc))
+    uri_exists = ghex_window_uri_exists(doc->file_name);
+    if(!hex_document_has_changed(doc) && uri_exists)
         return TRUE;
 
 	g_snprintf(msg, MESSAGE_LEN,
@@ -1068,7 +1170,7 @@ ghex_window_ok_to_close(GHexWindow *win)
 	gtk_widget_destroy(mbox);
 		
 	if(reply == GTK_RESPONSE_YES) {
-		if(doc->file_name == NULL) {
+		if(!uri_exists) {
 			if(!ghex_window_save_as(win)) {
 				return FALSE;
 			}
