@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* ghex.h - defines GHex ;)
 
-   Copyright (C) 1998 - 2002 Free Software Foundation
+   Copyright (C) 1998 - 2003 Free Software Foundation
 
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -28,8 +28,8 @@
 #include <gnome.h>
 
 #include <libgnomeprint/gnome-print.h>
-
-#include <libgnomeprint/gnome-print-master.h>
+#include <libgnomeprint/gnome-print-job.h>
+#include <gconf/gconf-client.h>
 
 #include <stdio.h>
 
@@ -51,7 +51,18 @@ G_BEGIN_DECLS
 
 #define NUM_MDI_MODES 4
 
-#define GHEX_URL "http://pluton.ijs.si/~jaka/gnome.html#GHEX"
+#define GHEX_URL "http://fish.homeunix.org/soft.html"
+
+#define GHEX_BASE_KEY                "/apps/ghex2"
+#define GHEX_PREF_FONT               "/font"
+#define GHEX_PREF_GROUP              "/group"
+#define GHEX_PREF_MAX_UNDO_DEPTH     "/maxundodepth"   
+#define GHEX_PREF_OFFSET_FORMAT      "/offsetformat"
+#define GHEX_PREF_OFFSETS_COLUMN     "/offsetscolumn"
+#define GHEX_PREF_PAPER              "/paper"
+#define GHEX_PREF_BOX_SIZE           "/boxsize"
+#define GHEX_PREF_DATA_FONT          "/datafont"
+#define GHEX_PREF_HEADER_FONT        "/headerfont"
 
 typedef struct _PropertyUI {
 	GtkWidget *pbox;
@@ -77,6 +88,8 @@ typedef struct _ReplaceDialog {
 	GtkWidget *replace, *replace_all, *next, *close;
 	GtkWidget *type_button[2];
 	
+	GtkHex_AutoHighlight *auto_highlight;
+	
 	gint search_type;
 } ReplaceDialog; 
 
@@ -89,9 +102,35 @@ typedef struct _FindDialog {
 	GtkWidget *f_next, *f_prev, *f_close;
 	GtkWidget *type_button[2];
 	
+	GtkHex_AutoHighlight *auto_highlight;
+	
 	gint search_type;
 } FindDialog;
 
+struct _AdvancedFindDialog {
+	GHexWindow *parent;
+	AdvancedFind_AddDialog *addDialog;
+
+	GtkWidget *window;
+	GtkWidget *hbox;
+	GtkWidget *vbox;
+	GtkListStore *list;
+	GtkWidget *tree;
+	GtkWidget *f_next, *f_prev;
+	GtkWidget *f_new, *f_remove;
+	GtkWidget *f_close;
+
+	gint search_type;
+};
+
+struct _AdvancedFind_AddDialog {
+	AdvancedFindDialog *parent;
+	GtkWidget *window;
+	GtkWidget *f_string;
+	GtkWidget *type_button[2];
+	GtkWidget *colour;
+};
+
 typedef struct _Converter {
 	GtkWidget *window;
 	GtkWidget *entry[5];
@@ -102,7 +141,7 @@ typedef struct _Converter {
 } Converter;
 
 typedef struct {
-	GnomePrintMaster *master;
+	GnomePrintJob *master;
 	GnomePrintContext *pc;
 	GnomePrintConfig *config;
 
@@ -163,13 +202,17 @@ extern gchar *search_type_label[2];
 
 extern gchar *geometry;
 
+extern GConfClient *gconf_client;
+
 /* creation of dialogs */
-FindDialog    *create_find_dialog    (void);
-ReplaceDialog *create_replace_dialog (void);
-JumpDialog    *create_jump_dialog    (void);
-Converter     *create_converter      (void);
-GtkWidget     *create_char_table     (void);
-PropertyUI    *create_prefs_dialog   (void);
+FindDialog              *create_find_dialog               (void);
+ReplaceDialog           *create_replace_dialog            (void);
+JumpDialog              *create_jump_dialog               (void);
+Converter               *create_converter                 (void);
+GtkWidget               *create_char_table                (void);
+PropertyUI              *create_prefs_dialog              (void);
+AdvancedFindDialog      *create_advanced_find_dialog      (GHexWindow *parent);
+void             delete_advanced_find_dialog      (AdvancedFindDialog *dialog);
 
 /* various ui convenience functions */
 void create_dialog_title   (GtkWidget *, gchar *);
@@ -184,11 +227,10 @@ GHexPrintJobInfo *ghex_print_job_info_new(HexDocument *doc, guint group_type);
 void ghex_print_job_info_destroy(GHexPrintJobInfo *pji);
 
 /* config stuff */
-void save_configuration    (void);
 void load_configuration    (void);
 
 /* hiding widgets on cancel or delete_event */
-gint delete_event_cb(GtkWidget *, GdkEventAny *);
+gint delete_event_cb(GtkWidget *, GdkEventAny *, GtkWindow *win);
 void cancel_cb(GtkWidget *, GtkWidget *);
 
 /* session managment */
@@ -211,6 +253,7 @@ extern GtkWidget *converter_get;
 void ghex_prefs_init (void);
 
 void find_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
+void advanced_find_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
 void replace_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
 void jump_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
 void set_byte_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
